// toShare reservation.js
///<reference path="user.js"/>
///<reference path="rest.js"/>
///<reference path="tooltip.js"/>
'use strict';

const tableId = Number(document.location.hash.substring(1));

const BULK_RESERVATION = true;
const SINGLE_RESERVATION = false;
async function initializePage(reservationsInst, reservationEditInst, itemMultiple){
	try{
		await atsReservationTables.restGet(tableId);
		const atsTable = atsReservationTables.findField('table_id',tableId);
		var pathname = (atsTable.table_custom === 0) ? '/reservation/' : '/custom/';
		pathname += atsTable.table_uri;
		if(location.pathname != pathname){
			location.href = pathname + location.hash;
		}else{
			await atsConfiguration.reflection();
			document.title = atsTable.table_name + ' - toShare';
			document.querySelector('h1').innerText =  atsTable.table_name;
			document.getElementById('table-item-name').innerText = atsTable.item_name;
			reservationEditInst.selectItem.multiple = itemMultiple && atsTable.item_bulk;
			await atsUsersTable.restGet(USER_ID_ALL);
			await atsItems.restGet(tableId);
			reservationEditInst.updateLoad();
			reservationsInst.updateLoad();
			reservationEditInst.inputDesc.placeholder = atsTable.table_desc;
		}
	}catch{
		location.href=location.origin;
	}
}
/** reserbation Base Class */
class reservations{
	constructor(){
		this.container = document.querySelector('section.reservation');
		this.durationDays = Number(this.container.dataset.durationDays);

		this.buttonPrev = this.container.querySelector('button.prev');
		this.buttonNext = this.container.querySelector('button.next');
		this.inputStartDate = this.container.querySelector('input.start-date');
		this.spanEndDate = this.container.querySelector('span.end-date');
		this.table = this.container.querySelector('table');
		this.defaultColGroup = this.table.querySelector('colgroup').cloneNode(true);
		this.defaultThead = this.table.querySelector('thead').cloneNode(true);
		this.defaultTbody = this.table.querySelector('tbody').cloneNode(true);

		this.onResize = this.onResize.bind(this);
		this.onStartDateChange = this.onStartDateChange.bind(this);
		this.onNextClick = this.onNextClick.bind(this);
		this.onPrevClick = this.onPrevClick.bind(this);
		this.onTableTouchstart = this.onTableTouchstart.bind(this);
		this.onTableTouchmove = this.onTableTouchmove.bind(this);
		this.onTableTouchend = this.onTableTouchend.bind(this);
		this.onWakeup = this.onWakeup.bind(this);
		this.nextTable = this.nextTable.bind(this);
		this.prevTable = this.prevTable.bind(this);

		window.addEventListener( 'resize', this.onResize);
		this.inputStartDate.addEventListener('change', this.onStartDateChange);
		this.buttonNext.addEventListener('click', this.onNextClick);
		this.buttonPrev.addEventListener('click', this.onPrevClick);
		this.table.addEventListener('touchstart', this.onTableTouchstart);
		this.table.addEventListener('touchmove', this.onTableTouchmove);
		this.table.addEventListener('touchend', this.onTableTouchend);
		atsSleepDetector.addWakeListener(this.onWakeup);

		const now = new Date;
		this.inputStartDate.value = formatDate(now, 'yyyy-MM-dd');

		this.startTouchTag = '';
		this.startTouchX = 0;
		this.startTouchScrollX = 0;
		this.currentTouchX = 0;
		this.currentTouchScrollX = 0;
		this.sensitivity = 10;
	}

	onResize(){
		const remPx = parseFloat(getComputedStyle(document.documentElement).fontSize);
		const viewHeight = window.innerHeight;
		const conteinerTop = this.container.getBoundingClientRect().top;
		const containerHeight = viewHeight - conteinerTop - 5 * remPx;
		this.table.style.maxHeight = containerHeight + 'px';
	}
	onStartDateChange(){
		this.table.scrollLeft = 0;
		this.updateLoad();
	}
	onNextClick(){
		const prevLeft = this.table.scrollLeft;
		this.table.scrollLeft += this.table.offsetWidth
		if(Math.abs(this.table.scrollLeft - prevLeft) <= 1){
			this.nextTable();
		}
	}
	onPrevClick(){
		const prevLeft = this.table.scrollLeft;
		this.table.scrollLeft -= this.table.offsetWidth
		if(Math.abs(this.table.scrollLeft - prevLeft) <= 1){
			this.prevTable();
		}
	}
	onTableTouchstart(event){
		this.startTouchTag = event.target.tagName;
		this.startTouchX = event.touches[0].clientX;
		this.startTouchScrollX = this.table.scrollLeft;
		this.onTableTouchmove(event);
	}
	onTableTouchmove(event){
		this.currentTouchX = event.touches[0].clientX;
		this.currentTouchScrollX = this.table.scrollLeft;
	}
	onTableTouchend(){
		switch(this.startTouchTag){
		case 'TH': break;
		case '': break;
		default: 
			const deltaScroll = this.currentTouchScrollX - this.startTouchScrollX;
			const delta = this.currentTouchX - this.startTouchX;
			if(Math.abs(deltaScroll)<=1){
				if(delta < -this.sensitivity){		this.nextTable();}
				else if(delta > this.sensitivity){	this.prevTable();}
			}
		}
		this.startTouchTag = '';
		this.startTouchX = this.currentTouchX;
		this.startTouchScrollX = this.currentTouchScrollX;
	}
	onWakeup(){
		if(confirm(messageDataRefresh))
			this.updateLoad();
	}
	nextTable(){
		var date = this.inputStartDate.valueAsDate;
		var drate = this.table.offsetWidth / this.table.scrollWidth *0.8;
		if(drate > 0.5){
			date.setDate(date.getDate() + this.durationDays);
		}else{
			date.setDate(date.getDate() + this.durationDays - this.durationDays * drate);
		}
		this.inputStartDate.valueAsDate = date;
		this.table.scrollLeft = 0;
		this.updateLoad();
	}
	prevTable(){
		var date = this.inputStartDate.valueAsDate;
		var drate = this.table.offsetWidth / this.table.scrollWidth *0.8;
		if(drate > 0.5){
			date.setDate(date.getDate() - this.durationDays);
		}else{
			date.setDate(date.getDate() - this.durationDays + this.durationDays * drate);
		}
		this.inputStartDate.valueAsDate = date;
		this.table.scrollLeft = 0;
		this.updateLoad();
		this.table.scrollLeft = this.table.scrollWidth - this.table.offsetWidth;
	}

	/**Update the entire reservation. */
	async updateLoad(){
		const startDate = this.inputStartDate.valueAsDate;
		startDate.setHours(0,0,0);
		const endDate = new Date(startDate);
		endDate.setDate(endDate.getDate() + this.durationDays );
		await atsCalendar.restGet(startDate, endDate);
		await atsReservations.restGet(tableId, startDate, endDate);

		endDate.setDate(endDate.getDate() -1);
		this.spanEndDate.textContent = formatDate(endDate,'～ MM/dd');

		const colGroup = this.defaultColGroup.cloneNode(true);
		const thead = this.defaultThead.cloneNode(true);
		const tbody = this.defaultTbody.cloneNode(true);

		this.updateThead(colGroup, thead);
		this.updateTbody(tbody);

		this.table.innerHTML = '';
		this.table.appendChild(colGroup);
		this.table.appendChild(thead);
		this.table.appendChild(tbody);
		this.updateReservations(tbody);
		this.onResize();
	}

	updateThead(colGroup, thead){
		const theadTr = thead.querySelector('thead tr');
		const theadDate = this.inputStartDate.valueAsDate;
		for(let i = 0; i<= this.durationDays-1; i++){
			const colGroupCol = document.createElement('col');
			const theadTh = document.createElement('th');
			colGroup.appendChild(colGroupCol);
			theadTh.scope="col";
			this.updateTheadTh(theadDate, colGroupCol, theadTh);

			theadTr.appendChild(theadTh);
			theadDate.setDate(theadDate.getDate() + 1);
		}
	}
	/** Draws the header calendar. Called from updateThead. */
	updateTheadTh(theadDate, colGroupCol, theadTh){
		const dateType = atsCalendar.dateType(theadDate); // lh sh
		if(dateType !== undefined){
			colGroupCol.classList.add(dateType);
			theadTh.classList.add(dateType);
		}
		theadTh.innerText = ((theadDate.getDate()===1) ? formatDate(theadDate, 'yyyy\nM/') : '') 
			+ formatDate(theadDate, 'd(week)');
	}

	updateTbody(tbody){
		const tbodyTrTemp = document.createElement('tr');
		const tbodyThTemp = document.createElement('th');
		tbodyThTemp.scope='row';
		tbodyTrTemp.appendChild(tbodyThTemp);
		for(let i = 0; i<= this.durationDays-1; i++){
			const tbodyTd = document.createElement('td');
			tbodyTrTemp.appendChild(tbodyTd);
		}

		for(const atsItem of atsItems.data){
			const tbodyTr = tbodyTrTemp.cloneNode(true);
			const tbodyTh = tbodyTr.firstChild;
			this.updateTbodyTh(atsItem, tbodyTh)
			tbody.appendChild(tbodyTr);
		}
	}
	/** Draws the item list. Called from updateTbody. */
	updateTbodyTh(atsItem, tbodyTh){
		tbodyTh.dataset.itemId = atsItem.item_id;
		tbodyTh.innerText = atsItem.item_name;
	}
	/** Draws the reservation status bar. Called from updateLoad. */
	updateReservations(tbody){
		const startDate = this.inputStartDate.valueAsDate;
		startDate.setHours(0,0,0);
		const endDate = new Date(startDate);
		endDate.setDate(endDate.getDate() + this.durationDays );

		const aDayWidth = tbody.rows[0].cells[1].getBoundingClientRect().width;
		const aDaySec = 86400;
		const scale = aDayWidth / aDaySec / 1000;

		for(const reservationItem of atsReservations.data){
			for(const tbodyTr of tbody.rows){
				if(Number(tbodyTr.cells[0].dataset.itemId)===reservationItem.item_id){
					// horizontal position & size
					const tbodyP = document.createElement('p');
					const tbodyDiv = document.createElement('div');
					const reservationStartDate = new Date(reservationItem.start_datetime);
					const reservationEndDate = new Date(reservationItem.end_datetime);
					if(reservationStartDate < startDate){reservationStartDate.setTime(startDate.getTime())}
					if(reservationEndDate > endDate){reservationEndDate.setTime(endDate.getTime())}
					const leftDate = new Date(reservationStartDate);
					leftDate.setHours(0,0,0);
					var pLeft = Math.floor((reservationStartDate.getTime() - leftDate.getTime()) * scale);
					var pWidth = Math.floor((reservationEndDate.getTime() - reservationStartDate.getTime()) * scale);
					if(pWidth===0){	pLeft -= 1;	pWidth = 2;	}
					tbodyDiv.style.left = pLeft +'px';
					tbodyP.style.width = pWidth + 'px';
					// information
					this.updateTbodyData(reservationItem, tbodyP);
					tbodyP.id = reservationItem.reservation_id;
					tbodyP.dataset.itemId = reservationItem.item_id;
					tbodyP.dataset.reservationId = reservationItem.reservation_id;
					tbodyP.dataset.bulkId = reservationItem.bulk_id;
					tbodyP.dataset.overlapGroupNo = 0;
					tbodyP.dataset.overlapCount = 0;
					// placed
					const colIndex = Math.floor((reservationStartDate.getTime() - startDate.getTime()) / 1000 / aDaySec) +1;
					tbodyDiv.appendChild(tbodyP);
					tbodyTr.cells[colIndex].appendChild(tbodyDiv);
//					tbodyTr.cells[colIndex].insertBefore(tbodyDiv, tbodyTr.cells[colIndex].firstChild)
				}
			}
		}
		// vertical overlap
		for(const tbodyTr of tbody.rows){
			const tbodyPs = tbodyTr.getElementsByTagName('p');
			const reservationItems = atsReservations.data.filter(d => d.item_id === Number(tbodyTr.cells[0].dataset.itemId));
			let groupCount = 0;
			for(const tbodyP of tbodyPs){
				const aItemIndex = reservationItems.findIndex(i => i.reservation_id === Number(tbodyP.dataset.reservationId));
				const aItem = reservationItems[aItemIndex];
				reservationItems.splice(aItemIndex,1);
				for(const oItem of reservationItems){
					if((aItem.start_datetime <= oItem.start_datetime) && (oItem.start_datetime < aItem.end_datetime)){
						const obodyP = tbodyPs.namedItem(oItem.reservation_id);
						if(Number(tbodyP.dataset.overlapGroupNo)===0){
							groupCount += 1;
							tbodyP.dataset.overlapGroupNo = groupCount;
						}
						obodyP.dataset.overlapGroupNo = tbodyP.dataset.overlapGroupNo;
						obodyP.dataset.overlapCount = Number(tbodyP.dataset.overlapCount) +1;
					}
				}
			}
			// vertical position & size
			for(let groupNo = 1; groupNo <= groupCount; groupNo++){
				const groupPs = Array.from(tbodyPs).filter(p => Number(p.dataset.overlapGroupNo) === groupNo);
				let overlapCount = 0;
				for(const tbodyP of groupPs){
					overlapCount = Math.max(overlapCount, Number(tbodyP.dataset.overlapCount));
				}
				for(const tbodyP of groupPs){
					switch (overlapCount){
					case 1:
						tbodyP.style.top = Number(tbodyP.dataset.overlapCount) *1.6 + 'em';
						tbodyP.style.height = '1.5em';
						break;
					case 2:
						tbodyP.style.top = Number(tbodyP.dataset.overlapCount) *1.1 + 'em';
						tbodyP.style.height = '1em';
						break;
					case 3:
						tbodyP.style.top = Number(tbodyP.dataset.overlapCount) *0.8 + 'em';
						tbodyP.style.height = '0.7em';
						break;
					case 4:
						tbodyP.style.top = Number(tbodyP.dataset.overlapCount) *0.7 + 'em';
						tbodyP.style.height = '0.6em';
						break;
					}
				}
			}
		}
	}

	/** Description of the reservation item. Called from updateReservations. */
	updateTbodyData(reservationItem, tbodyP){
		const reservationStartDate = new Date(reservationItem.start_datetime);
		const reservationEndDate = new Date(reservationItem.end_datetime);
		tbodyP.innerText = reservationItem.reservation_desc + '\n'
			+ reservationItem.user_name + '\n'
			+ formatDate(reservationStartDate, 'yyyy/MM/dd HH:mm から\n')
			+ formatDate(reservationEndDate, 'yyyy/MM/dd HH:mm まで');
	}

}

const MINIMUM_UNIT_TIME = 'time';
const MINIMUM_UNIT_DATE = 'date';
class todayView{
	constructor(reservations, reservationEdit, minimumUnit){
		// reservation = reservation class or extend reservation class
		this.reservations = reservations;
		this.reservationEdit = reservationEdit;
		this.minimumUnitTime = minimumUnit === MINIMUM_UNIT_TIME;
		this.container = document.querySelector('div.today-view');
		this.todayArea = this.container.querySelector('div.today-area');
		this.todayTitle = this.container.querySelector('div.today-title label');
		this.selectTitle = this.container.querySelector('div.today-title select');
		this.todayList = this.container.querySelector('div.today-list');
		this.buttonCancel = this.container.querySelector('button.cancel');
		this.defaultTable = this.container.querySelector('table').cloneNode(true);
		this.defaultTableTh = this.defaultTable.querySelector('thead th').cloneNode(true);
		this.defaultTableTr = this.defaultTable.querySelector('tbody tr').cloneNode(true);

		this.today = new Date();

		this.onClose = this.onClose.bind(this);
		this.onOutsideClick = this.onOutsideClick.bind(this);
		this.onSelectTitleChage = this.onSelectTitleChage.bind(this);
		this.onTheadThClick = this.onTheadThClick.bind(this);
		this.onTrClick = this.onTrClick.bind(this);

		this.container.addEventListener('click', this.onOutsideClick);
		this.buttonCancel.addEventListener('click', this.onClose);
		this.selectTitle.addEventListener('change', this.onSelectTitleChage);
	}
	onTheadThClick(event){
		this.updateList(new Date(event.target.dataset.tooltip));
		this.container.style.display = 'block';
	}
	onClose(){
		this.container.style.display = 'none';
	}
	onOutsideClick(event){
		if(event.currentTarget===event.target){
			this.onClose();
		}
	}
	onSelectTitleChage(){
		this.updateList(this.today);
	}
	sameDay(date1, date2) {
		const d1 = new Date(date1);
		const d2 = new Date(date2);

		const year1 = d1.getFullYear();
		const month1 = d1.getMonth();
		const day1 = d1.getDate();

		const year2 = d2.getFullYear();
		const month2 = d2.getMonth();
		const day2 = d2.getDate();

		return year1 === year2 && month1 === month2 && day1 === day2;
	}
	async updateList(today){
		this.today = today;
		this.todayTitle.innerText = formatDate(today, 'yyyy年MM月dd日');

		const startDate = this.today;
		startDate.setHours(0,0,0);
		const endDate = new Date(startDate);
		endDate.setHours(23, 59, 59);
		const todayReservation = new restReservations();
		if(this.selectTitle.selectedIndex == 0){
			await todayReservation.restGet(tableId, startDate, endDate);
		}else{
			await todayReservation.restGetLastUses(tableId, startDate);
		}
		this.defaultTable.innerHTML = '';
		this.todayList.innerHTML = '';
		for(const atsItem of atsItems.data){
			const itemTable = this.defaultTable.cloneNode(true);
			const itemTableTh = this.defaultTableTh.cloneNode(true);
			itemTable.appendChild(itemTableTh);
			this.updateTh(atsItem, itemTableTh);
			for(const reservationItem of todayReservation.data){
				if(reservationItem.item_id===atsItem.item_id){
					const itemTableTr = this.defaultTableTr.cloneNode(true);
					this.updateTr(reservationItem, itemTableTr);
					itemTable.appendChild(itemTableTr);
				}
			}
			this.todayList.appendChild(itemTable);
		}
	}
	updateTh(atsItem, itemTableTh){
		itemTableTh.innerText = atsItem.item_name;
		itemTableTh.dataset.itemId = atsItem.item_id;
		itemTableTh.dataset.tooltip = atsItem.item_desc;
		itemTableTh.addEventListener('click', this.reservationEdit.onTbodyThClick);
		itemTableTh.addEventListener('click', this.onClose);
	}
	updateTr(reservationItem, itemTableTr){
		const td = itemTableTr.querySelectorAll("td");
		const reservationStartDate = new Date(reservationItem.start_datetime);
		const reservationEndDate = new Date(reservationItem.end_datetime);
		let startDateFormat = '';
		let endDateFormat = '';
		if(this.minimumUnitTime){
			startDateFormat = this.sameDay(this.today, reservationStartDate) ? 'HH:mm' : '';
			endDateFormat = this.sameDay(this.today, reservationEndDate) ? ' ～ HH:mm' : ' ～ yyyy/MM/dd';
		}else{
			startDateFormat = this.sameDay(this.today, reservationStartDate) ? '本日' : '';
			endDateFormat = this.sameDay(this.today, reservationEndDate) ? '' : ' ～ yyyy/MM/dd';
		}
		td[0].textContent = formatDate(reservationStartDate, startDateFormat) 
							+ formatDate(reservationEndDate, endDateFormat);
		td[1].innerText = reservationItem.user_name;
		td[2].innerText = reservationItem.reservation_desc;

		if((this.selectTitle.selectedIndex != 0)&&(this.today < reservationEndDate)){
			itemTableTr.classList.add('previous');
		};

		itemTableTr.dataset.reservationId = reservationItem.reservation_id;
		itemTableTr.dataset.startDate = reservationItem.start_datetime;
		itemTableTr.addEventListener('click', this.onTrClick);
	}
	async onTrClick(event){
		const reservationId = event.currentTarget.dataset.reservationId;
		const inputStartDate = this.reservations.inputStartDate.valueAsDate;
		const startDate = new Date(event.currentTarget.dataset.startDate)
		startDate.setHours(9,0,0);
		inputStartDate.setHours(9,0,0);
		if(startDate < inputStartDate){
			this.reservations.inputStartDate.valueAsDate = startDate;
			await this.reservations.updateLoad();
		}
		this.reservationEdit.editOpen(Number(reservationId));
		this.onClose();
	}

}

const NOT_SELECTED = -1;
class reservationEdit{
	constructor(reservations, minimumUnit){
		// reservation = reservation class or extend reservation class
		// minimumUnit = 'date' or 'time'
		this.reservations = reservations;
		this.minimumUnitTime = minimumUnit === MINIMUM_UNIT_TIME;
		this.container = document.querySelector('section.edit');

		this.inner = this.container.querySelector('.inner');
		this.reservationId = this.container.querySelector('input.reservation-id');
		this.bulkId = this.container.querySelector('input.bulk-id');
		this.inputDesc = this.container.querySelector('input.desc');
		this.selectUser = this.container.querySelector('select.user');
		this.selectItem = this.container.querySelector('select.item');
		this.selectTypical = this.container.querySelector('select.typical');
		this.inputStartDate = this.container.querySelector('input.start-date');
		this.inputEndDate = this.container.querySelector('input.end-date');
		if(this.minimumUnitTime){
			this.inputStartTime = this.container.querySelector('input.start-time');
			this.inputEndTime = this.container.querySelector('input.end-time');
		}
		this.buttonAdd = this.container.querySelector('button.add');
		this.buttonFix = this.container.querySelector('button.fix');
		this.buttonDel = this.container.querySelector('button.del');
		this.buttonOpen = this.container.querySelector('.button-open');
		this.buttonClose = this.container.querySelector('.button-close');

		this.onItemChange = this.onItemChange.bind(this);
		this.onDescChange = this.onDescChange.bind(this);
		this.onTypicalChange = this.onTypicalChange.bind(this);
		this.onDateTimeInput = this.onDateTimeInput.bind(this);
		this.onDateTimeFocusin = this.onDateTimeFocusin.bind(this);
		this.onDateTimeFocusout = this.onDateTimeFocusout.bind(this);
		this.onExecute = this.onExecute.bind(this);
		this.onInnerOpen = this.onInnerOpen.bind(this);
		this.onInnerClose = this.onInnerClose.bind(this);
		this.onTBodyClick = this.onTBodyClick.bind(this);
		this.onTbodyThClick = this.onTbodyThClick.bind(this);
		this.onTbodyDataClick = this.onTbodyDataClick.bind(this);

		this.selectItem.addEventListener('change', this.onItemChange);
		this.inputDesc.addEventListener('input', this.onDescChange);
		this.selectTypical.addEventListener('change', this.onTypicalChange);

		this.inputStartDate.addEventListener('input', this.onDateTimeInput);
		this.inputEndDate.addEventListener('input', this.onDateTimeInput);
		if(this.minimumUnitTime){
			this.inputStartTime.addEventListener('input', this.onDateTimeInput);
			this.inputEndTime.addEventListener('input', this.onDateTimeInput);
		}
		this.originStartDate = 
		this.originStartTime = 
		this.originEndDate = 
		this.originEndTime = ''
		this.inputStartDate.addEventListener('focusin', this.onDateTimeFocusin);
		this.inputEndDate.addEventListener('focusin', this.onDateTimeFocusin);
		if(this.minimumUnitTime){
			this.inputStartTime.addEventListener('focusin', this.onDateTimeFocusin);
			this.inputEndTime.addEventListener('focusin', this.onDateTimeFocusin);
		}
		this.inputStartDate.addEventListener('focusout', this.onDateTimeFocusout);
		this.inputEndDate.addEventListener('focusout', this.onDateTimeFocusout);
		if(this.minimumUnitTime){
			this.inputStartTime.addEventListener('focusout', this.onDateTimeFocusout);
			this.inputEndTime.addEventListener('focusout', this.onDateTimeFocusout);
		}

		this.buttonAdd.addEventListener('click', this.onExecute);
		this.buttonFix.addEventListener('click', this.onExecute);
		this.buttonDel.addEventListener('click', this.onExecute);
		this.buttonOpen.addEventListener('click', this.onInnerOpen);
		this.buttonClose.addEventListener('click', this.onInnerClose);
	}

	updateLoad(){
		this.reservationId.value = 0;
		this.bulkId.value = 0;

		this.selectUser.innerHTML = '';
		for(const atsUser of atsUsersTable.data){
			let userOption = document.createElement('option');
			userOption.dataset.userId = atsUser.user_id;
			userOption.innerText = atsUser.user_name;
			this.selectUser.appendChild(userOption);
		};

		this.selectUser.selectedIndex = NOT_SELECTED;
		for(const selectOption of this.selectUser){
			if(Number(selectOption.dataset.userId)===atsCurrentUserId){
				this.selectUser.selectedIndex = selectOption.index;
				break;
			}
		}

		this.selectItem.innerHTML = '';
		for(const atsItem of atsItems.data){
			let ItemOption = document.createElement('option');
			ItemOption.dataset.itemId = atsItem.item_id;
			ItemOption.innerText = atsItem.item_name;
			this.selectItem.appendChild(ItemOption);
		};
		this.selectItem.selectedIndex = -1;
		this.inputDesc.value = '';
		const now = new Date;
		this.inputStartDate.value = 
		this.inputEndDate.value = formatDate(now, 'yyyy-MM-dd');
		if(this.minimumUnitTime){
			this.inputStartTime.value = '08:45:00';
			this.inputEndTime.value = '17:00:00';
		}
		this.buttonAdd.disabled = true;
		this.buttonFix.disabled = true;
		this.buttonDel.disabled = true;
	}
	isDateTimeInconsistency(){
		var result = (this.inputStartDate.value > this.inputEndDate.value);
		if(this.minimumUnitTime){
			result = ((this.inputStartDate.value === this.inputEndDate.value)
				  &&  (this.inputStartTime.value >= this.inputEndTime.value))
				  || result;
		}
		return result;
	}
	buttonDisableUpdate(){
		if(Number(this.reservationId.value)===0){
			this.buttonAdd.disabled = (this.inputDesc.value==='')
									||(this.isDateTimeInconsistency())
									||(this.selectItem.selectedIndex===-1)
									||(this.selectUser.selectedIndex===-1);
			this.buttonFix.disabled = true;
			this.buttonDel.disabled = true;
		}else{
			// fix or del
			this.buttonAdd.disabled = true;
			this.buttonFix.disabled = (this.inputDesc.value==='')
									||(this.isDateTimeInconsistency())
									||(this.bulkId.dataset.count != this.selectItem.selectedOptions.length);
			this.buttonDel.disabled = false;
		}
	}
	tableSelectUpdate(){
		for(const selectOption of this.selectItem){
			const thItem = this.reservations.table.tBodies[0].rows[selectOption.index].cells[0];
			if(selectOption.selected){
				if(!thItem.classList.contains('select')){thItem.classList.add('select')};
			}else{
				if(thItem.classList.contains('select')){thItem.classList.remove('select')};
			}
		};
	}

	onItemChange(){
		this.tableSelectUpdate();
		this.buttonDisableUpdate();
	}

	onDescChange(){
		this.buttonDisableUpdate();
	}

	onTypicalChange(event){
		const now = new Date;
		const selected =  event.target[event.target.selectedIndex];

		if(this.minimumUnitTime){
			switch(selected.dataset.typical){
			case 'today':
				this.inputStartDate.value = 
				this.inputEndDate.value = formatDate(now, 'yyyy-MM-dd');
				this.inputStartTime.value = '08:45:00';
				this.inputEndTime.value = '17:00:00';
				break;
			case 'am':
				this.inputStartDate.value = 
				this.inputEndDate.value = formatDate(now, 'yyyy-MM-dd');
				this.inputStartTime.value = '08:45:00';
				this.inputEndTime.value = '12:00:00';
				break;
			case 'pm':
				this.inputStartDate.value = 
				this.inputEndDate.value = formatDate(now, 'yyyy-MM-dd');
				this.inputStartTime.value = '12:45:00';
				this.inputEndTime.value = '17:00:00';
				break;
			case '1h':
				const startData = new Date;
				const endData = new Date;
				startData.setMinutes(Math.round(startData.getMinutes() /10) *10);
				endData.setMinutes(Math.round(endData.getMinutes() /30) *30);
				endData.setHours(endData.getHours() +1);
				this.inputStartDate.value = formatDate(startData, 'yyyy-MM-dd');
				this.inputEndDate.value = formatDate(endData, 'yyyy-MM-dd');
				this.inputStartTime.value = formatDate(startData, 'HH:mm:00');
				this.inputEndTime.value = formatDate(endData, 'HH:mm:00');
				break;
			} 
		}else{
			switch(selected.dataset.typical){
				case 'today':
					this.inputStartDate.value = 
					this.inputEndDate.value = formatDate(now, 'yyyy-MM-dd');
					break;
				case 'week':
					const weekData = new Date;
					weekData.setDate(weekData.getDate() + 7);
					this.inputStartDate.value = formatDate(now, 'yyyy-MM-dd');
					this.inputEndDate.value = formatDate(weekData, 'yyyy-MM-dd');
					break;
				case 'month':
					const monthData = new Date;
					monthData.setMonth(monthData.getMonth() + 1);
					this.inputStartDate.value = formatDate(now, 'yyyy-MM-dd');
					this.inputEndDate.value = formatDate(monthData, 'yyyy-MM-dd');
					break;
			}
		}
		highlight(this.inputStartDate);
		highlight(this.inputEndDate);
		if(this.minimumUnitTime){
			highlight(this.inputStartTime);
			highlight(this.inputEndTime);
		}
	}

	onDateTimeFocusin(){
		this.originStartDate = this.inputStartDate.value==='' ? this.originStartDate : this.inputStartDate.value;
		this.originEndDate =  this.inputEndDate.value==='' ? this.originEndDate : this.inputEndDate.value;
		if(this.minimumUnitTime){
			this.originStartTime = this.inputStartTime.value;
			this.originEndTime = this.inputEndTime.value;
		}
	}
	onDateTimeInput(event){
		if(!isHoverSupported || (event.target.value==='')){
			this.checkInputDateTimeConsistency();
		}
		this.buttonDisableUpdate();
	}
	onDateTimeFocusout(event){
		this.checkDateTimeConsistency(event);
		this.checkInputDateTimeConsistency();
	}
	checkInputDateTimeConsistency(){
		if((this.inputStartDate.valueAsDate===null)||(this.inputEndDate.valueAsDate===null)){
			this.inputStartDate.value = this.originStartDate;
			this.inputEndDate.value =  this.originEndDate;
			if(this.minimumUnitTime){
				this.inputStartTime.value = this.originStartTime;
				this.inputEndTime.value = this.originEndTime;
			}
		}
	}
	checkDateTimeConsistency(event){
		for(const selectOption of this.selectTypical){
			if(selectOption.dataset.typical === 'antonym'){
				this.selectTypical.selectedIndex = selectOption.index;
			}
		}
		const startDateTime = this.inputStartDate.valueAsDate;
		const endDateTime =	 this.inputEndDate.valueAsDate;
		if(this.minimumUnitTime){
			const startTimeSect = this.inputStartTime.value;
			const startTimeSplit = startTimeSect.split(':');
			const endTimeSect = this.inputEndTime.value;
			const endTimeSplit = endTimeSect.split(':');
			startDateTime.setHours(startTimeSplit[0], startTimeSplit[1]);
			endDateTime.setHours(endTimeSplit[0], endTimeSplit[1]);
		}

		if(startDateTime > endDateTime){
			if(event.target.classList.contains('start-date')){
				endDateTime.setFullYear(startDateTime.getFullYear(),startDateTime.getMonth(),startDateTime.getDate());
				highlight(this.inputEndDate);
			}else{
				startDateTime.setFullYear(endDateTime.getFullYear(),endDateTime.getMonth(),endDateTime.getDate());
				highlight(this.inputStartDate);
			}
		}
		if(this.minimumUnitTime){
			if(startDateTime > endDateTime){
				if(event.target.classList.contains('start-time')){
					endDateTime.setHours(startDateTime.getHours(),startDateTime.getMinutes(),startDateTime.getSeconds());
					highlight(this.inputEndTime);
				}else{
					startDateTime.setHours(endDateTime.getHours(),endDateTime.getMinutes(),endDateTime.getSeconds());
					highlight(this.inputStartTime);
				}
			}
		}
		this.inputStartDate.value = formatDate(startDateTime, 'yyyy-MM-dd');
		this.inputEndDate.value = formatDate(endDateTime, 'yyyy-MM-dd');
		if(this.minimumUnitTime){
			this.inputStartTime.value = formatDate(startDateTime, 'HH:mm:ss');
			this.inputEndTime.value = formatDate(endDateTime, 'HH:mm:ss');
		}
		this.buttonDisableUpdate();
	}
	getSelectItemIds(){
		const itemIds=[];
		for(const selectOption of this.selectItem.selectedOptions){
			itemIds.push(Number(selectOption.dataset.itemId));
		}
		return itemIds;
	}

	async onExecute(event){
		const startDateTime = this.inputStartDate.valueAsDate;
		const endDateTime =	 this.inputEndDate.valueAsDate;
		if(!this.minimumUnitTime){
			startDateTime.setHours(0, 0, 1);
			endDateTime.setHours(23, 59, 59);
		}else{
			const startTimeSect = this.inputStartTime.value;
			const startTimeSplit = startTimeSect.split(':');
			const endTimeSect = this.inputEndTime.value;
			const endTimeSplit = endTimeSect.split(':');
			startDateTime.setHours(startTimeSplit[0], startTimeSplit[1]);
			endDateTime.setHours(endTimeSplit[0], endTimeSplit[1]);
		}
		const selectItem = this.selectItem;
		const selectUser = this.selectUser;

		switch(event.target){
			case this.buttonAdd:{
				const doubleBook = await this.checkDoubleBook(startDateTime, endDateTime);
				if(!doubleBook){
					try{
						await atsReservations.restPost(
							this.getSelectItemIds(),
							Number(selectUser[selectUser.selectedIndex].dataset.userId),
							startDateTime,
							endDateTime,
							this.inputDesc.value
						)
					}catch(e){confirm(e.message);}
				}
				break;
			}
			case this.buttonFix:{
				const doubleBook = await this.checkDoubleBook(startDateTime, endDateTime);
				if(!doubleBook){
					const userId = (selectUser.selectedIndex === NOT_SELECTED)
								? Number(this.selectUser.dataset.userId) 
								: Number(selectUser[selectUser.selectedIndex].dataset.userId);
					try{
						await atsReservations.restPut(
							Number(this.reservationId.value),
							this.getSelectItemIds(),
							userId,
							startDateTime,
							endDateTime,
							this.inputDesc.value
						)
					}catch(e){confirm(e.message);}
				}
				break;
			}
			case this.buttonDel:{
				try{
					await atsReservations.restDelete(
						Number(this.reservationId.value)
					)
				}catch(e){confirm(e.message);}
				break;
			}
			default: confirm('onExecute tagert unknown');
		}
		this.updateLoad();
		this.reservations.updateLoad();
		this.onInnerClose();
	}

	async checkDoubleBook(startDateTime, endDateTime){
		const targetItemIds = this.getSelectItemIds();
		const targetReservationId = Number(this.reservationId.value);
		const targetBulkId = Number(this.bulkId.value);
		const checkReservation = new restReservations();
		await checkReservation.restGet(tableId, startDateTime, endDateTime);
		var doubleBook = false;
		for(const reservationItem of checkReservation.data){
			if(targetItemIds.includes(reservationItem.item_id)){
				if(((targetBulkId ===0)&&(targetReservationId !== reservationItem.reservation_id))
				|| ((targetBulkId !==0)&&(targetBulkId !== reservationItem.bulk_id))){
					doubleBook = true;
					break;
				}
			}
		}
		if(doubleBook){doubleBook = ! confirm('重複または隣接しています。実行しますか？')}
		return doubleBook;
	}

	// reservation tbody th click. and todayview caption click.
	onTbodyThClick(event){
		for(const selectOption of this.selectItem){
			if(Number(selectOption.dataset.itemId) === Number(event.currentTarget.dataset.itemId)){
				this.onInnerOpen();
				selectOption.selected = !selectOption.selected;
				highlight(this.selectItem);
			}
		}
		this.tableSelectUpdate();
		this.buttonDisableUpdate();
	}
	clearCurrentAndSelect(){
		this.selectItem.selectedIndex = -1;
		for(const item of this.reservations.table.querySelectorAll('tr td p')){
			if(item.classList.contains('current')){item.classList.remove('current');}
			if(item.classList.contains('select')){item.classList.remove('select');}
		}
		for(const item of this.reservations.table.querySelectorAll('tr th')){
			if(item.classList.contains('current')){item.classList.remove('current');}
			if(item.classList.contains('select')){item.classList.remove('select');}
		}
	}

	onTbodyDataClick(event){
		if(!event.target.classList.contains('current')){
			this.editOpen(Number(event.currentTarget.dataset.reservationId));
		}else{
			this.editCancel();
		}
	}

	editOpen(reservationId){
		const reservationItem = atsReservations.findField('reservation_id', reservationId);
		const reservationStartDate = new Date(reservationItem.start_datetime);
		const reservationEndDate = new Date(reservationItem.end_datetime);

		this.clearCurrentAndSelect();
		const targetBulkId = reservationItem.bulk_id;
		if(targetBulkId === 0){
			for(const item of this.reservations.table.querySelectorAll('tr td p')){
				if(Number(item.dataset.reservationId)===reservationId){item.classList.add('current')}
			}
		}else{// bulk reservation
			for(const item of this.reservations.table.querySelectorAll('tr td p')){
				if(Number(item.dataset.bulkId)===targetBulkId){item.classList.add('current')}
			}
		}

		this.reservationId.value = reservationItem.reservation_id;
		this.bulkId.value = reservationItem.bulk_id;
		this.selectUser.dataset.userId = reservationItem.user_id;
		this.selectUser.selectedIndex = NOT_SELECTED;
		for(const selectOption of this.selectUser){
			if(Number(selectOption.dataset.userId)===reservationItem.user_id){
				this.selectUser.selectedIndex = selectOption.index;
			}
		}
		if(targetBulkId === 0){
			for(const selectOption of this.selectItem){
				if(Number(selectOption.dataset.itemId)===reservationItem.item_id){
					selectOption.selected =true;
				};
			}
		}else{// bulk reservation
			for(const item of this.reservations.table.querySelectorAll('tr td p')){
				if(Number(item.dataset.bulkId)===targetBulkId){
					for(const selectOption of this.selectItem){
						if(Number(selectOption.dataset.itemId)===Number(item.dataset.itemId)){
							selectOption.selected =true;
						};
					}
				}
			}
		}
		this.bulkId.dataset.count = this.selectItem.selectedOptions.length;
		this.inputDesc.value = reservationItem.reservation_desc;
		this.inputStartDate.value = formatDate(reservationStartDate, 'yyyy-MM-dd');
		this.inputEndDate.value = formatDate(reservationEndDate, 'yyyy-MM-dd');
		if(this.minimumUnitTime){
			this.inputStartTime.value = formatDate(reservationStartDate, 'HH:mm:ss');
			this.inputEndTime.value = formatDate(reservationEndDate, 'HH:mm:ss');
		}
		this.onInnerOpen();
		this.tableSelectUpdate();
		this.buttonDisableUpdate();
		highlight(this.container);
	}
	onTBodyClick(event){
		if((event.target.tagName === 'TD')
		&&(Number(this.reservationId.value)!==0)){
			this.editCancel();
		}
	}
	editCancel(){
		this.clearCurrentAndSelect();
		this.updateLoad();
		highlight(this.container);
		this.onInnerClose();
	}
	onInnerOpen(){
		this.inner.classList.remove('hidden');
		this.buttonOpen.classList.add('hidden');
		this.buttonClose.classList.remove('hidden');
		this.tableSelectUpdate();
		this.reservations.onResize();
	}
	onInnerClose(){
		this.inner.classList.add('hidden');
		this.buttonOpen.classList.remove('hidden');
		this.buttonClose.classList.add('hidden');
		this.reservations.onResize();
	}
}
