// toShare tooltip.js
'use strict';

// tooltip function
const tooltipText = document.createElement('div');
tooltipText.classList.add('tooltipText');
document.body.appendChild(tooltipText);

/** show mouseover tooltip from data-tooltip on HTMLElement  */
function showTooltip(event){
	if(event.target.dataset.tooltip !== undefined){
		tooltipText.innerText = (event.target.dataset.tooltip==='') ?
			event.target.innerText : event.target.dataset.tooltip;
		tooltipText.style.left = (event.pageX + 16) + 'px';
		tooltipText.style.top = (event.pageY + 16) + 'px';
		tooltipText.style.display = 'inline-block';
		tooltipText.dataset.show = 'view';
	}else{
		hiddenTooltip();
	}
};
function hiddenTooltip(){
	if(tooltipText.dataset.show === 'view'){
		tooltipText.dataset.show = 'hidden';
		setTimeout(()=> {
			if(tooltipText.dataset.show === 'hidden'){
				tooltipText.style.display = 'none';
			}
		}, 700);
	}
}

/**
 * highlight HTMLElement
 * @param {HTMLElement} element
 */
function highlight(element){
	element.classList.add('highlightStart');
	setTimeout(()=>{
		element.classList.remove('highlightStart');
		element.classList.add('highlightEnd');
	}, 100);
	setTimeout(()=>{
		element.classList.remove('highlightEnd');
	}, 500);
}

/** copy tooltip text to clipboard function */
function onCopyClipboard(event){
	if((tooltipText.style.display==='inline-block')
	&&(document.getSelection().toString()==='')){
		event.clipboardData.setData('text/plain', tooltipText.innerText);
		event.preventDefault();
	}
}

/** wake reload  */
class sleepDetector {
	constructor(){
		this.listeners = [];
		this.lastCheckTime = Date.now();
		this.intervalId = null;
		this.sleepDetected = false;
//		this.timeoutId = null;
		this.idleDetected = false;
		this.idleCount = 0;
		this.checkIdleWakeup = this.checkIdleWakeup.bind(this);
	}
	/**
	 * @param {function} listener - wakeup listener function
	 * @example
	 * // silentReload predefined function
	 * .addWakeListener(silentReload);
	 * // confirmReload predefined function
	 * .addWakeListener(confirmReload);
	 * // execute user function
	 * .confirmReload(function)
	 */
	addWakeListener(listener){
		this.listeners.push(listener);
		this.startInterval();
	}
	startInterval(){
		if (this.intervalId !== null) return;

		this.lastCheckTime = Date.now();
		this.sleepDetected = false;
		this.idleCount = 0;
		this.intervalId = setInterval(() => {
			const currentTime = Date.now();
			const timeDifference = currentTime - this.lastCheckTime;
			this.lastCheckTime = currentTime;
			this.idleCount = this.idleCount + 500;
			if (this.idleCount > 1000*60*5)this.idleDetected = true;			// idle 5min
			if (timeDifference > 1000*60)this.sleepDetected = true;				// sleep 1min
			if ((timeDifference < 600) && this.sleepDetected)this.onWakeup();	// sleep detected wake
		}, 500);

		window.addEventListener('mousemove', this.checkIdleWakeup);
		window.addEventListener('scroll', this.checkIdleWakeup);
	}
	checkIdleWakeup(){
		this.idleCount = 0;
		if(this.idleDetected)this.onWakeup();	// idle detected wake
	}
	onWakeup(){
		this.idleDetected = false;
		this.sleepDetected = false;
		this.listeners.forEach(listener => listener());
	}
}
const messageDataRefresh = '表示更新しますか？';
function confirmReload(){
	if(confirm(messageDataRefresh))
		location.reload();
}
function silentReload(){
	location.reload();
}
// regist eventlistener (tooltip without smartphone)
const isHoverSupported = matchMedia('(hover: hover)').matches || matchMedia('(hover: hover)').media === 'none';
if(isHoverSupported){
	window.addEventListener('mouseover', showTooltip);
	window.addEventListener('mousemove', showTooltip);
	window.addEventListener('touchend', hiddenTooltip);
	document.addEventListener('copy', onCopyClipboard);
}

const atsSleepDetector = new sleepDetector();
// ex1:) atsSleepDetector.addWakeListener(confirmReload);
// ex2:) atsSleepDetector.addWakeListener(silentReload);
