// toShare co_host/inoutboard.js
///<reference path="../ats/rest.js"/>
///<reference path="../ats/tooltip.js"/>
'use strict';

const hostId = Number(document.location.hash.substring(1));
if (hostId == 0){location.href=location.origin;}

const mainTag = document.querySelector('main');
const sectionInoutboardTemp = document.querySelector('section.inoutboard').cloneNode(true);
const trUserTemp = document.querySelector('section.inoutboard tbody tr').cloneNode(true);
const buttonUpdatePage = document.getElementById('update-page');
const spanCyclicEffective =  document.getElementById('cyclic-effective');
const divPeriodLayer = document.querySelector('div.period-layer');
const buttonUpdate = document.getElementById('update');
const buttonCyclic = document.getElementById('cyclic');
const buttonsingle = document.getElementById('single');
var intervalId = null;
sectionInoutboardTemp.querySelector('tbody').innerHTML = '';

/**
* toShare Inoutboard API CoHosts Inoutboards
*/
class restCoHostsInoutboards extends restClient {
	/**
	 * @override
	 */
	constructor(){
		super(`${basePath}/co_hosts/inoutboards`, 'inoutboards');
	};
	/**
	 * Get Inoutboards
	 * @override
	 * @returns 
	 */
	restGet(){
		const query = new URLSearchParams({'host_id': hostId});
		this.url = `${this.baseURL}?${query}`;
		return super.restGet()
	}
};

/**
 * toShare Inoutboard API CoHosts Attendance
 */
class restCoHostsAttendance extends restClient {
	/**
	 * @override
	 */
	constructor(){
		super(`${basePath}/co_hosts/attendance`, 'attendance');
	};
	/**
	 * Get Attendance
	 * @override
	 * @returns 
	 */
	restGet(){
		const query = new URLSearchParams({'host_id': hostId});
		this.url = `${this.baseURL}?${query}`;
		return super.restGet()
	}
};
/** An instance of restCoHostsInoutboards (Atelier Arika toShare CoHosts API) */
const atsCoHostsInoutboards = new restCoHostsInoutboards();
/** An instance of restCoHostsAttendance (Atelier Arika toShare CoHosts API) */
const atsCoHostsAttendance = new restCoHostsAttendance();

async function initializaPage(){
	try{
		await atsConfiguration.reflection();
		await atsCoHosts.restGet();
		await atsCoHostsInoutboards.restGet()

		const atsHost = atsCoHosts.findField('host_id',hostId);
		const h1Tag = document.querySelector('h1');
		h1Tag.innerHTML = '&#x27B0; 行先掲示:';
		h1Tag.innerText +=  atsHost.host_name;
		document.title = h1Tag.innerText + ' - toShare'

		const cycleEffect = localStorage.getItem('inoutboardCyclicEffect') == 'true';
		if(cycleEffect){
			onButtonCyclicClick();
		}else{
			onButtonSingleClick();
		}
		attendanceUpdateLoad();
	}catch(e){
		confirm('接続に異常がありました。\n' + e.message);
		location.href=location.origin;
	}
}

async function attendanceUpdateLoad(){
	await atsCoHostsAttendance.restGet();
	const doubleBuffer = document.createElement('main');
	for(const atsInoutboard of atsCoHostsInoutboards.data){
		doubleBuffer.appendChild(getSectionInoutboard(atsInoutboard))
	}
	mainTag.innerHTML = doubleBuffer.innerHTML;
	buttonUpdatePage.innerHTML = formatDate(new Date(), '&#x21BB;更新 (HH:mm)');
	highlight(buttonUpdatePage);
}

function getSectionInoutboard(atsInoutboard){
	const sectionInoutboard = sectionInoutboardTemp.cloneNode(true);
	sectionInoutboard.querySelector('caption').innerText = atsInoutboard.board_name;
	const inoutboardTbody = sectionInoutboard.querySelector('tbody');
	for(const atsUser of atsCoHostsAttendance.data){
		if(atsInoutboard.board_id == atsUser.board_id){
			const trUser = trUserTemp.cloneNode(true);
			trUser.dataset.userId = atsUser.user_id;

			const td = trUser.querySelectorAll('td');
			td[0].innerText = atsUser.user_name;
			td[0].classList.add(atsUser.clock_status ? 'clock-in': 'clock-out');
			td[1].innerText = atsUser.whereabouts;
			td[2].innerText = atsUser.description;

			inoutboardTbody.appendChild(trUser);
		}
	}
	return sectionInoutboard;
}

function openPeriodWindow(){
	divPeriodLayer.style.display = 'flex';
}
function closePeriodWindow(){
	divPeriodLayer.style.display = 'none';
}
function onOutsideClick(event){
	if(event.currentTarget == event.target)closePeriodWindow();
}

function onButtonUpdateClick(){
	attendanceUpdateLoad();
	closePeriodWindow();
}
function onButtonCyclicClick(){
	if(!intervalId)
		intervalId = setInterval(attendanceUpdateLoad, 1000*60*5);
	closePeriodWindow();
	spanCyclicEffective.style.display = 'inline';
	localStorage.setItem('inoutboardCyclicEffect', true);
}
function onButtonSingleClick(){
	clearInterval(intervalId);
	intervalId = null;
	closePeriodWindow();
	spanCyclicEffective.style.display = 'none';
	localStorage.setItem('inoutboardCyclicEffect', false);
}
function onWakeup(){
	if(!intervalId){
		if(confirm(messageDataRefresh))
			attendanceUpdateLoad();
	}else{	attendanceUpdateLoad();}
}

buttonUpdatePage.addEventListener('click', openPeriodWindow);
divPeriodLayer.addEventListener('click', onOutsideClick);
buttonUpdate.addEventListener('click', onButtonUpdateClick);
buttonCyclic.addEventListener('click', onButtonCyclicClick);
buttonsingle.addEventListener('click', onButtonSingleClick);

initializaPage();
atsSleepDetector.addWakeListener(onWakeup);
