// toShare items.js
///<reference path="../ats/rest.js"/>
'use strict';

const sectionItems = document.querySelector('section.items');
const buttonNew = sectionItems.querySelector('button.new');
const buttonUp = sectionItems.querySelector('button.up');
const buttonDown = sectionItems.querySelector('button.down');
const tableItem = sectionItems.querySelector('table');
const tbodyItem = sectionItems.querySelector('tbody');
const trTemplateItem = tbodyItem.firstElementChild.cloneNode(true);
const divEditLayer = document.querySelector('div.edit-layer');
const buttonCancel = divEditLayer.querySelector('button.cancel');
const inputItemId = divEditLayer.querySelector('input.item-id');
const inputItemName = divEditLayer.querySelector('input.item-name');
const inputItemDesc = divEditLayer.querySelector('input.item-desc');
const checkboxDisable = divEditLayer.querySelector('input.disable');
const labelDisable = divEditLayer.querySelector('label.disable');
const buttonAdd = divEditLayer.querySelector('button.add');
const buttonFix = divEditLayer.querySelector('button.fix');
const buttonDel = divEditLayer.querySelector('button.del');
const UNSELECT = -1;
let orderStartIndex = UNSELECT;
let orderEndIndex = UNSELECT;
let memoMouseOutside = false;

async function setTitle(tableId){
	await atsReservationTables.restGet(TABLE_ID_ALL, WITHDISABLED);
	const atsTable = atsReservationTables.findField('table_id',tableId);
	const pageTitle = `${atsTable.table_name} - ${atsTable.item_name}`;
	document.title = `${pageTitle} - toShare`
	document.querySelector('h1').innerText =  pageTitle;
}

async function updateItemList(){
	await atsItems.restGet(atsItems.tableId ,WITHDISABLED);
	tbodyItem.innerHTML = '';
	for(const atsItem of atsItems.data){
		const itemTr = trTemplateItem.cloneNode(true);
		itemTr.dataset.itemId = atsItem.item_id;
		if(atsItem.disable){itemTr.classList.add('disable');}
		const td = itemTr.querySelectorAll("td");
		td[0].innerText = atsItem.item_name;
		td[1].innerText = atsItem.item_desc;
		td[2].textContent = (atsItem.disable)? '非表示': '';

		td[0].addEventListener('click', onTdItemClick);
		td[1].addEventListener('click', onTdItemClick);
		td[2].addEventListener('click', onTdItemClick);
		td[3].addEventListener('click', onTdOrderClick);
		tbodyItem.appendChild(itemTr);
	}
}

function redrawSelected(){
	buttonUp.disabled = (orderStartIndex===UNSELECT)||(orderStartIndex===0);
	buttonDown.disabled = (orderEndIndex===UNSELECT)||(orderEndIndex===tbodyItem.rows.length-1);
	for(const tbodyTr of tbodyItem.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('select')){tbodyTr.classList.remove('select');}
	}
	if(orderStartIndex != UNSELECT){
		for(let index=orderStartIndex; index <= orderEndIndex; index++){
			tbodyItem.rows[index].classList.add('select');
		}
	}
}

function onNewEdit(){
	for(const tbodyTr of tbodyItem.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
	}
	divEditLayer.style.display = 'flex';
	inputItemId.value = "0";
	inputItemName.value = "";
	inputItemDesc.value = "";
	inputItemName.focus();
	checkboxDisable.style.visibility = 'hidden';
	labelDisable.style.visibility = 'hidden';
	buttonAdd.disabled = true;
	buttonFix.disabled = true;
	buttonDel.disabled = true;
}

async function onOrderExecute(event){
	let itemIds = [];
	for(let index =orderStartIndex; index <= orderEndIndex; index++){
		itemIds.push(atsItems.data[index].item_id);
	}
	switch(event.target){
		case buttonUp:{
			itemIds.push(atsItems.data[orderStartIndex-1].item_id);
			try{
				await atsItems.restOrderPut(itemIds)
				orderStartIndex -=1;
				orderEndIndex -=1;
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonDown:{
			itemIds.unshift(atsItems.data[orderEndIndex+1].item_id);
			try{
				await atsItems.restOrderPut(itemIds)
				orderStartIndex +=1;
				orderEndIndex +=1;
			}catch(e){confirm(e.message);}
			break;
		}
		default: confirm('onExecute tagert unknown');
	}
	await updateItemList();
	redrawSelected();
}

function onTdItemClick(event){
	orderStartIndex = UNSELECT;
	orderEndIndex = UNSELECT;
	redrawSelected();
	if(!event.currentTarget.parentElement.classList.contains('current')){
		for(const tbodyTr of tbodyItem.querySelectorAll('tr')){
			if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
		}
		event.target.parentElement.classList.add('current');
		divEditLayer.style.display = 'flex';
		const atsItem = atsItems.findField('item_id', Number(event.currentTarget.parentElement.dataset.itemId));
		inputItemId.value = atsItem.item_id;
		inputItemName.value = atsItem.item_name;
		inputItemName.select();
		inputItemName.focus();
		inputItemDesc.value = atsItem.item_desc;
		checkboxDisable.style.visibility = 'visible';
		labelDisable.style.visibility = 'visible';
		checkboxDisable.checked = (atsItem.disable===1);
		buttonAdd.disabled = true;
		buttonFix.disabled = true;
		buttonDel.disabled = false;
	}else{
		event.currentTarget.parentElement.classList.remove('current');
		divEditLayer.style.display = 'none';
	}
}

function onTdOrderClick(event){
	let index = event.currentTarget.parentElement.rowIndex -1;
	if((orderStartIndex > index)||(orderStartIndex===UNSELECT)){
		orderStartIndex = index;
		orderEndIndex = index;
	}else if(orderStartIndex===index){
		orderStartIndex = UNSELECT;
		orderEndIndex = UNSELECT;
	}else if(orderStartIndex < index){
		orderEndIndex = index;
	};
	redrawSelected();
}

function onOutsideClick(event){
	if(event.currentTarget===event.target){
		if(memoMouseOutside){
			onCancel();
		}else{
			 memoMouseOutside = true;
		}
	}else{
		memoMouseOutside = false;
	}
}
function onCancel(){
	for(const tbodyTr of tbodyItem.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
	}
	divEditLayer.style.display = 'none';
}

function onItemModified(){
	if(Number(inputItemId.value)===0){
		buttonAdd.disabled = (inputItemName.value==='');
		buttonFix.disabled = true;
		buttonDel.disabled = true;
	}else{
		// fix or del
		buttonAdd.disabled = true;
		buttonFix.disabled = false;
		buttonDel.disabled = false;
	}
}
async function onExecute(event){
	switch(event.target){
		case buttonAdd:{
			try{
				await atsItems.restPost(
					inputItemName.value,
					inputItemDesc.value
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonFix:{
			try{
				await atsItems.restPut(
					Number(inputItemId.value),
					inputItemName.value,
					inputItemDesc.value,
					(checkboxDisable.checked)? 1: 0
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonDel:{
			try{
				await atsItems.restDelete(
					Number(inputItemId.value)
				)
			}catch(e){confirm(e.message);}
			break;
		}
		default: confirm('onExecute tagert unknown');
	}
	divEditLayer.style.display = 'none';
	updateItemList();
}


tbodyItem.innerHTML = '';
atsConfiguration.reflection();

atsItems.tableId = Number(document.location.hash.substring(1));
setTitle(atsItems.tableId);
updateItemList();

divEditLayer.addEventListener('mousedown', onOutsideClick);
divEditLayer.addEventListener('mouseup', onOutsideClick);
buttonNew.addEventListener('click', onNewEdit);
buttonUp.addEventListener('click', onOrderExecute);
buttonDown.addEventListener('click', onOrderExecute);
buttonCancel.addEventListener('click', onCancel);
inputItemName.addEventListener('input', onItemModified);
inputItemDesc.addEventListener('input', onItemModified);
checkboxDisable.addEventListener('input', onItemModified);
buttonAdd.addEventListener('click', onExecute);
buttonFix.addEventListener('click', onExecute);
buttonDel.addEventListener('click', onExecute);
