// toShare users.js
'use strict';
///<reference path="../ats/rest.js"/>

const sectionUsers = document.querySelector('section.users');
const buttonNew = sectionUsers.querySelector('button.new');
const buttonUp = sectionUsers.querySelector('button.up');
const buttonDown = sectionUsers.querySelector('button.down');
const tableUser = sectionUsers.querySelector('table');
const tbodyUser = sectionUsers.querySelector('tbody');
const trTemplateUser = tbodyUser.firstElementChild.cloneNode(true);
const divEditLayer = document.querySelector('div.edit-layer');
const buttonCancel = divEditLayer.querySelector('button.cancel');
const inputUserId = divEditLayer.querySelector('input.user-id');
const inputUserAccount = divEditLayer.querySelector('input.user-account');
const inputUserName = divEditLayer.querySelector('input.user-name');
const inputUserDesc = divEditLayer.querySelector('input.user-desc');
const inputUserSeries = divEditLayer.querySelector('input.user-series');
const checkboxDisable = divEditLayer.querySelector('input.disable');
const labelDisable = divEditLayer.querySelector('label.disable');
const buttonAdd = divEditLayer.querySelector('button.add');
const buttonFix = divEditLayer.querySelector('button.fix');
const buttonDel = divEditLayer.querySelector('button.del');
const UNSELECT = -1;
let memoMouseOutside = false;
let orderExecute = false;
let orderStartIndex = UNSELECT;
let orderEndIndex = UNSELECT;

async function initializePage(){
	tbodyUser.innerHTML = '';
	await atsConfiguration.reflection();
	updateUserList();
	accountPlaceholderSet();
}

function accountPlaceholderSet(){
	atsConfiguration.restGet(['Color style', 'Account Placeholder']).then(()=>{
		inputUserAccount.placeholder = atsConfiguration.valueFromKey('Account Placeholder');
	});
}

async function updateUserList(){
	await atsUsersTable.restGet(USER_ID_ALL, WITHDISABLED);
	tbodyUser.innerHTML = '';
	for(const atsUser of atsUsersTable.data){
		const userTr = trTemplateUser.cloneNode(true);
		const td = userTr.querySelectorAll("td");
		userTr.dataset.userId = atsUser.user_id;
		if(atsUser.disable){userTr.classList.add('disable');}
		td[0].innerText = atsUser.user_account;
		td[1].innerText = atsUser.user_name;
		td[2].innerText = atsUser.user_desc;
		td[3].innerText = atsUser.user_series;
		td[4].textContent = (atsUser.disable)? '非表示' : '';

		td[0].addEventListener('click', onTdUserClick);
		td[1].addEventListener('click', onTdUserClick);
		td[2].addEventListener('click', onTdUserClick);
		td[3].addEventListener('click', onTdUserClick);
		td[4].addEventListener('click', onTdUserClick);
		td[5].addEventListener('click', onTdOrderClick);
		tbodyUser.appendChild(userTr);
	}
}

function redrawSelected(){
	buttonUp.disabled = (orderStartIndex===UNSELECT)||(orderStartIndex===0);
	buttonDown.disabled = (orderEndIndex===UNSELECT)||(orderEndIndex===tbodyUser.rows.length-1);
	for(const tbodyTr of tbodyUser.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('select')){tbodyTr.classList.remove('select');}
	}
	if(orderStartIndex != UNSELECT){
		for(let index=orderStartIndex; index <= orderEndIndex; index++){
			tbodyUser.rows[index].classList.add('select');
		}
	}
}

function onNewEdit(){
	for(const tbodyTr of tbodyUser.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
	}
	divEditLayer.style.display = 'flex';
	inputUserId.value = "0";
	inputUserAccount.value = "";
	inputUserName.value = "";
	inputUserDesc.value = "";
	inputUserSeries.value = "";
	inputUserAccount.focus();
	checkboxDisable.style.visibility = 'hidden';
	labelDisable.style.visibility = 'hidden';
	buttonAdd.disabled = true;
	buttonFix.disabled = true;
	buttonDel.disabled = true;
}

async function onOrderExecute(event){
	let userIds = [];
	for(let index =orderStartIndex; index <= orderEndIndex; index++){
		userIds.push(atsUsersTable.data[index].user_id);
	}
	switch(event.target){
		case buttonUp:{
			userIds.push(atsUsersTable.data[orderStartIndex-1].user_id);
			try{
				await atsUsersTable.restOrderPut(userIds)
				orderStartIndex -=1;
				orderEndIndex -=1;
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonDown:{
			userIds.unshift(atsUsersTable.data[orderEndIndex+1].user_id);
			try{
				await atsUsersTable.restOrderPut(userIds)
				orderStartIndex +=1;
				orderEndIndex +=1;
			}catch(e){confirm(e.message);}
			break;
		}
		default: confirm('onExecute tagert unknown');
	}
	await updateUserList();
	redrawSelected();
}

function onTdUserClick(event){
	orderStartIndex = UNSELECT;
	orderEndIndex = UNSELECT;
	redrawSelected();
	if(!event.currentTarget.parentElement.classList.contains('current')){
		for(const tbodyTr of tbodyUser.querySelectorAll('tr')){
			if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
		}
		event.currentTarget.parentElement.classList.add('current');
		divEditLayer.style.display = 'flex';
		const atsUser = atsUsersTable.findField('user_id', Number(event.currentTarget.parentElement.dataset.userId));
		inputUserId.value = atsUser.user_id;
		inputUserAccount.value = atsUser.user_account;
		inputUserAccount.select();
		inputUserAccount.focus();
		inputUserName.value = atsUser.user_name;
		inputUserDesc.value = atsUser.user_desc;
		inputUserSeries.value = atsUser.user_series;
		checkboxDisable.style.visibility = 'visible';
		labelDisable.style.visibility = 'visible';
		checkboxDisable.checked = (atsUser.disable===1);
		buttonAdd.disabled = true;
		buttonFix.disabled = true;
		buttonDel.disabled = false;
	}else{
		event.currentTarget.parentElement.classList.remove('current');
		divEditLayer.style.display = 'none';
	}
}

function onTdOrderClick(event){
	let index = event.currentTarget.parentElement.rowIndex -1;
	if((orderStartIndex > index)||(orderStartIndex===UNSELECT)){
		orderStartIndex = index;
		orderEndIndex = index;
	}else if(orderStartIndex===index){
		orderStartIndex = UNSELECT;
		orderEndIndex = UNSELECT;
	}else if(orderStartIndex < index){
		orderEndIndex = index;
	};
	redrawSelected();
}

function onOutsideClick(event){
	if(event.currentTarget===event.target){
		if(memoMouseOutside){
			onCancel();
		}else{
			 memoMouseOutside = true;
		}
	}else{
		memoMouseOutside = false;
	}
}
function onCancel(){
	for(const tbodyTr of tbodyUser.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
	}
	divEditLayer.style.display = 'none';
}
function onUserModified(){
	if(Number(inputUserId.value)===0){
		buttonAdd.disabled = (inputUserAccount.value==='') || (inputUserName.value==='');
		buttonFix.disabled = true;
		buttonDel.disabled = true;
	}else{
		// fix or del
		buttonAdd.disabled = true;
		buttonFix.disabled = false;
		buttonDel.disabled = false;
	}
}
async function onExecute(event){
	switch(event.target){
		case buttonAdd:{
			try{
				await atsUsersTable.restPost(
					inputUserAccount.value,
					inputUserName.value,
					inputUserDesc.value,
					inputUserSeries.value
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonFix:{
			try{
				await atsUsersTable.restPut(
					Number(inputUserId.value),
					inputUserAccount.value,
					inputUserName.value,
					inputUserDesc.value,
					inputUserSeries.value,
					(checkboxDisable.checked)? 1: 0
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonDel:{
			try{
				await atsUsersTable.restDelete(
					Number(inputUserId.value)
				)
			}catch(e){confirm(e.message);}
			break;
		}
		default: confirm('onExecute tagert unknown');
	}
	divEditLayer.style.display = 'none';
	updateUserList();
}

initializePage();
divEditLayer.addEventListener('mousedown', onOutsideClick);
divEditLayer.addEventListener('mouseup', onOutsideClick);
buttonNew.addEventListener('click', onNewEdit);
buttonUp.addEventListener('click', onOrderExecute);
buttonDown.addEventListener('click', onOrderExecute);
buttonCancel.addEventListener('click', onCancel);
inputUserAccount.addEventListener('input', onUserModified);
inputUserName.addEventListener('input', onUserModified);
inputUserDesc.addEventListener('input', onUserModified);
inputUserSeries.addEventListener('input', onUserModified);
checkboxDisable.addEventListener('change', onUserModified);
buttonAdd.addEventListener('click', onExecute);
buttonFix.addEventListener('click', onExecute);
buttonDel.addEventListener('click', onExecute);
