// toShare whereabouts.js
'use strict';
///<reference path="../ats/rest.js"/>

const sectionWhereabouts = document.querySelector('section.whereabouts');
const buttonNew = sectionWhereabouts.querySelector('button.new');
const buttonUp = sectionWhereabouts.querySelector('button.up');
const buttonDown = sectionWhereabouts.querySelector('button.down');
const tableWhereabouts = sectionWhereabouts.querySelector('table');
const tbodyWhereabouts = sectionWhereabouts.querySelector('tbody');
const trTemplateWhereabouts = tbodyWhereabouts.firstElementChild.cloneNode(true);
const divEditLayer = document.querySelector('div.edit-layer');
const buttonCancel = divEditLayer.querySelector('button.cancel');
const inputWhereaboutsId = divEditLayer.querySelector('input.whereabouts-id');
const selectWhereaboutsOpt = divEditLayer.querySelector('select.whereabouts-opt');
const inputWhereaboutsName = divEditLayer.querySelector('input.whereabouts-name');
const checkboxDisable = divEditLayer.querySelector('input.disable');
const labelDisable = divEditLayer.querySelector('label.disable');
const buttonAdd = divEditLayer.querySelector('button.add');
const buttonFix = divEditLayer.querySelector('button.fix');
const buttonDel = divEditLayer.querySelector('button.del');
const UNSELECT = -1;
const OPT_GROUP = 1;
const OPT_NAME = 0;
let orderStartIndex = UNSELECT;
let orderEndIndex = UNSELECT;

async function updateWhereaboutsList(){
	await atsWhereabouts.restGet(WITHDISABLED);
	tbodyWhereabouts.innerHTML = '';
	let optGroup = false;
	for(const atsAWhereabouts of atsWhereabouts.data){
		const whereaboutsTr = trTemplateWhereabouts.cloneNode(true);
		whereaboutsTr.dataset.whereaboutsId = atsAWhereabouts.whereabouts_id;
		if(atsAWhereabouts.disable){whereaboutsTr.classList.add('disable');}
		const td = whereaboutsTr.querySelectorAll("td");
		if(atsAWhereabouts.optGroup == OPT_GROUP){
			optGroup = optGroup || !atsAWhereabouts.disable;
			td[0].classList.add('optGroup');
		}else if(optGroup){
			td[0].classList.add('child');
		};

		td[0].innerText = atsAWhereabouts.whereabouts;
		td[1].textContent = (atsAWhereabouts.disable)? '非表示': '';

		td[0].addEventListener('click', onTdWhereaboutsClick);
		td[1].addEventListener('click', onTdWhereaboutsClick);
		td[2].addEventListener('click', onTdOrderClick);
		tbodyWhereabouts.appendChild(whereaboutsTr);
	}
}

function redrawSelected(){
	buttonUp.disabled = (orderStartIndex===UNSELECT)||(orderStartIndex===0);
	buttonDown.disabled = (orderEndIndex===UNSELECT)||(orderEndIndex===tbodyWhereabouts.rows.length-1);
	for(const tbodyTr of tbodyWhereabouts.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('select')){tbodyTr.classList.remove('select');}
	}
	if(orderStartIndex != UNSELECT){
		for(let index=orderStartIndex; index <= orderEndIndex; index++){
			tbodyWhereabouts.rows[index].classList.add('select');
		}
	}
}


function onNewEdit(){
	for(const tbodyTr of tbodyWhereabouts.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
	}
	divEditLayer.style.display = 'flex';
	inputWhereaboutsId.value = "0";
	selectWhereaboutsOpt.selectedIndex = OPT_NAME;
	onWhereaboutsSelect();
	inputWhereaboutsName.value = "";
	inputWhereaboutsName.focus();
	checkboxDisable.style.visibility = 'hidden';
	labelDisable.style.visibility = 'hidden';
	buttonAdd.disabled = true;
	buttonFix.disabled = true;
	buttonDel.disabled = true;
}

async function onOrderExecute(event){
	let whereaboutsIds = [];
	for(let index =orderStartIndex; index <= orderEndIndex; index++){
		whereaboutsIds.push(atsWhereabouts.data[index].whereabouts_id);
	}
	switch(event.target){
		case buttonUp:{
			whereaboutsIds.push(atsWhereabouts.data[orderStartIndex-1].whereabouts_id);
			try{
				await atsWhereabouts.restOrderPut(whereaboutsIds)
				orderStartIndex -=1;
				orderEndIndex -=1;
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonDown:{
			whereaboutsIds.unshift(atsWhereabouts.data[orderEndIndex+1].whereabouts_id);
			try{
				await atsWhereabouts.restOrderPut(whereaboutsIds)
				orderStartIndex +=1;
				orderEndIndex +=1;
			}catch(e){confirm(e.message);}
			break;
		}
		default: confirm('onExecute tagert unknown');
	}
	await updateWhereaboutsList();
	redrawSelected();
}

function onTdWhereaboutsClick(event){
	orderStartIndex = UNSELECT;
	orderEndIndex = UNSELECT;
	redrawSelected();
	if(!event.currentTarget.parentElement.classList.contains('current')){
		for(const tbodyTr of tbodyWhereabouts.querySelectorAll('tr')){
			if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
		}
		event.target.parentElement.classList.add('current');
		divEditLayer.style.display = 'flex';
		const atsAWhereabouts = atsWhereabouts.findField('whereabouts_id', Number(event.currentTarget.parentElement.dataset.whereaboutsId));
		inputWhereaboutsId.value = atsAWhereabouts.whereabouts_id;
		selectWhereaboutsOpt.selectedIndex = atsAWhereabouts.optGroup;
		onWhereaboutsSelect();
		inputWhereaboutsName.value = atsAWhereabouts.whereabouts;
		inputWhereaboutsName.select();
		inputWhereaboutsName.focus();
		checkboxDisable.style.visibility = 'visible';
		labelDisable.style.visibility = 'visible';
		checkboxDisable.checked = (atsAWhereabouts.disable===1);
		buttonAdd.disabled = true;
		buttonFix.disabled = true;
		buttonDel.disabled = false;
	}else{
		event.currentTarget.parentElement.classList.remove('current');
		divEditLayer.style.display = 'none';
	}
}

function onTdOrderClick(event){
	let index = event.currentTarget.parentElement.rowIndex -1;
	if((orderStartIndex > index)||(orderStartIndex===UNSELECT)){
		orderStartIndex = index;
		orderEndIndex = index;
	}else if(orderStartIndex===index){
		orderStartIndex = UNSELECT;
		orderEndIndex = UNSELECT;
	}else if(orderStartIndex < index){
		orderEndIndex = index;
	};
	redrawSelected();
}

function onOutsideClick(event){
	if(event.currentTarget===event.target){
		onCancel();
	}
}
function onCancel(){
	for(const tbodyTr of tbodyWhereabouts.querySelectorAll('tr')){
		if(tbodyTr.classList.contains('current')){tbodyTr.classList.remove('current');}
	}
	divEditLayer.style.display = 'none';
}
function onWhereaboutsSelect(){
	switch(selectWhereaboutsOpt.selectedIndex){
	case OPT_GROUP:
		inputWhereaboutsName.placeholder ='グループ名';
		break;
	case OPT_NAME:
		inputWhereaboutsName.placeholder ='所在名';
		break;
	}
}

function onWhereaboutsModified(){
	if(Number(inputWhereaboutsId.value)===0){
		buttonAdd.disabled = (inputWhereaboutsName.value==='');
		buttonFix.disabled = true;
		buttonDel.disabled = true;
	}else{
		// fix or del
		buttonAdd.disabled = true;
		buttonFix.disabled = false;
		buttonDel.disabled = false;
	}
}
async function onExecute(event){
	switch(event.target){
		case buttonAdd:{
			try{
				await atsWhereabouts.restPost(
					inputWhereaboutsName.value,
					selectWhereaboutsOpt.selectedIndex
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonFix:{
			try{
				await atsWhereabouts.restPut(
					Number(inputWhereaboutsId.value),
					inputWhereaboutsName.value,
					selectWhereaboutsOpt.selectedIndex,
					(checkboxDisable.checked)? 1: 0
				)
			}catch(e){confirm(e.message);}
			break;
		}
		case buttonDel:{
			try{
				await atsWhereabouts.restDelete(
					Number(inputWhereaboutsId.value)
				)
			}catch(e){confirm(e.message);}
			break;
		}
		default: confirm('onExecute tagert unknown');
	}
	divEditLayer.style.display = 'none';
	updateWhereaboutsList();
}


tbodyWhereabouts.innerHTML = '';
atsConfiguration.reflection();

updateWhereaboutsList();

divEditLayer.addEventListener('click', onOutsideClick);
buttonNew.addEventListener('click', onNewEdit);
buttonUp.addEventListener('click', onOrderExecute);
buttonDown.addEventListener('click', onOrderExecute);
buttonCancel.addEventListener('click', onCancel);
selectWhereaboutsOpt.addEventListener('change', onWhereaboutsSelect);
selectWhereaboutsOpt.addEventListener('change', onWhereaboutsModified);
inputWhereaboutsName.addEventListener('input', onWhereaboutsModified);
checkboxDisable.addEventListener('input', onWhereaboutsModified);
buttonAdd.addEventListener('click', onExecute);
buttonFix.addEventListener('click', onExecute);
buttonDel.addEventListener('click', onExecute);

